<?php

namespace Smsto\Hubspot\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;
use Smsto\Hubspot\Models\Settings;

class CreateTimelineJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    /**
     * The validated data
     */
    protected $validated;

    /**
     * The settings id
     */
    protected $settings_id;

    /**
     * The object id
     */
    protected $associatedObjectId;

    /**
     * Create a new job instance.
     *
     * @param array $validated
     * @param string $api_key
     * @return void
     */
    public function __construct(array $validated, int $settings_id, int $associatedObjectId)
    {
        $this->validated = $validated;
        $this->settings_id = $settings_id;
        $this->associatedObjectId = $associatedObjectId;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $settings = Settings::find($this->settings_id);
        $response = Http::withHeaders(['X-Smsto-Integration-Name' => 'hubspot'])->withToken($settings->api_key)->asJson()->acceptJson()->get('https://api.sms.to/message/' . $this->validated['messageId']);
        $response = json_decode($response, true);
        if (empty($response)) {
            Log::error('Message not found in callback.', $this->validated);
            abort(404, 'Message not found in callback. ID: ' . $this->validated['messageId']);
        }

        $timeline = [
            'inputFields' => [
                'to' => $response['to'],
                'message' => $response['message'],
                'sender_id' => $response['sender_id'],
                'status' => $response['status'],
            ],
            'object' => [
                'objectId' => $this->associatedObjectId,
            ],
        ];

        $settings->createTimelineEvent($timeline);
        if ($response['status'] !== 'REJECTED') {
            return response([
                "outputFields" => [
                    'hs_execution_state' => "SUCCESS",
                    "myOutput" => "Message processed"
                ]
            ]);
        } else {
            return response([
                "outputFields" => [
                    'hs_execution_state' => "FAIL_CONTINUE",
                    "myOutput" => "Something went wrong"
                ]
            ]);
        }
    }
}
